@@ -69,7 +69,7 @@
  * Paul Mackerras (paulus@cs.anu.edu.au).
  */
 
-/* $Id: if_ppp.c,v 1.12 1994/05/24 23:00:05 paulus Exp $ */
+/* $Id: if_ppp.c,v 1.13 1994/06/08 00:38:03 paulus Exp $ */
 /* from if_sl.c,v 1.11 84/10/04 12:54:47 rick Exp */
 
 #include "ppp.h"
@@ -436,7 +436,9 @@ pppwrite(tp, uio, flag)
 	}
 	if (uio->uio_resid >= MCLBYTES / 2)
 	    MCLGET(m, M_DONTWAIT);
-	len = MIN(M_TRAILINGSPACE(m), uio->uio_resid);
+	len = M_TRAILINGSPACE(m);
+	if (len > uio->uio_resid)
+	    len = uio->uio_resid;
 	if (error = uiomove(mtod(m, u_char *), len, uio)) {
 	    m_freem(m0);
 	    return (error);
@@ -937,7 +939,8 @@ pppstart(tp)
 		    for (nleft = n; nleft > 0; nleft -= cc) {
 			if ((cc = RB_CONTIGPUT(&tp->t_out)) == 0)
 			    break;
-			cc = min (cc, nleft);
+			if (cc > nleft)
+			    cc = nleft;
 			bcopy((char *)start + n - nleft, tp->t_out.rb_tl, cc);
 			tp->t_out.rb_tl = RB_ROLLOVER(&tp->t_out,
 						      tp->t_out.rb_tl + cc);
@@ -1531,7 +1534,8 @@ pppdumpm(m0, pktlen)
 	u_char *rptr = (u_char *)m->m_data;
 
 	if (pktlen > 0) {
-	    l = min(l, pktlen);
+	    if (l > pktlen)
+		l = pktlen;
 	    pktlen -= l;
 	}
 	while (l--) {
