@@ -423,7 +423,9 @@ pppwrite(tp, uio, flag)
 	}
 	if (uio->uio_resid >= MCLBYTES / 2)
 	    MCLGET(m, M_DONTWAIT);
-	len = MIN(M_TRAILINGSPACE(m), uio->uio_resid);
+	len = M_TRAILINGSPACE(m);
+	if (len > uio->uio_resid)
+	    len = uio->uio_resid;
 	if (error = uiomove(mtod(m, u_char *), len, uio)) {
 	    m_freem(m0);
 	    return (error);
@@ -918,27 +920,8 @@ pppstart(tp)
 			break;
 		n = cp - start;
 		if (n) {
-#ifndef	RB_LEN
 		    /* NetBSD (0.9 or later), 4.3-Reno or similar. */
 		    ndone = n - b_to_q(start, n, &tp->t_outq);
-#else
-#ifdef	NetBSD
-		    /* NetBSD with 2-byte ring buffer entries */
-		    ndone = rb_cwrite(&tp->t_out, start, n);
-#else
-		    /* 386BSD, FreeBSD */
-		    int cc, nleft;
-		    for (nleft = n; nleft > 0; nleft -= cc) {
-			if ((cc = RB_CONTIGPUT(&tp->t_out)) == 0)
-			    break;
-			cc = min (cc, nleft);
-			bcopy((char *)start + n - nleft, tp->t_out.rb_tl, cc);
-			tp->t_out.rb_tl = RB_ROLLOVER(&tp->t_out,
-						      tp->t_out.rb_tl + cc);
-		    }
-		    ndone = n - nleft;
-#endif	/* NetBSD */
-#endif	/* RB_LEN */
 		    len -= ndone;
 		    start += ndone;
 		    sc->sc_bytessent += ndone;
@@ -1525,7 +1508,8 @@ pppdumpm(m0, pktlen)
 	u_char *rptr = (u_char *)m->m_data;
 
 	if (pktlen > 0) {
-	    l = min(l, pktlen);
+	    if (l > pktlen)
+		l = pktlen;
 	    pktlen -= l;
 	}
 	while (l--) {
