@@ -282,8 +282,6 @@ lcp_extcode(f, code, id, inp, len)
 	LCPDEBUG((LOG_INFO, "lcp: Echo-Request, Rcvd id %d", id));
 	magp = inp;
 	PUTLONG(lcp_gotoptions[f->unit].magicnumber, magp);
-	if (len < CILEN_LONG)
-	    len = CILEN_LONG;
 	fsm_sdata(f, ECHOREP, id, inp, len);
 	break;
     
@@ -816,7 +814,7 @@ lcp_nakci(f, p, len)
     if (f->state != OPENED) {
 	*go = try;
 	if (looped_back && try.numloops % lcp_warnloops == 0)
-	    LCPDEBUG((LOG_INFO, "The line appears to be looped back."));
+	    syslog(LOG_WARNING, "Serial line appears to be looped back.");
     }
 
     return 1;
@@ -1483,7 +1481,8 @@ void LcpLinkFailure (f)
 {
     if (f->state == OPENED) {
         syslog (LOG_NOTICE, "Excessive lack of response to LCP echo frames.");
-        lcp_lowerdown(f->unit);		/* Reset connection */
+	lcp_close(f->unit);
+	phase = PHASE_TERMINATE;
     }
 }
 
@@ -1561,12 +1560,16 @@ lcp_received_echo_reply (f, id, inp, len)
     u_long magic;
 
     /* Check the magic number - don't count replies from ourselves. */
-    if (len < CILEN_LONG)
+    if (len < 4) {
+	syslog(LOG_DEBUG, "lcp: received short Echo-Reply, length %d", len);
 	return;
+    }
     GETLONG(magic, inp);
     if (lcp_gotoptions[f->unit].neg_magicnumber
-	&& magic == lcp_gotoptions[f->unit].magicnumber)
+	&& magic == lcp_gotoptions[f->unit].magicnumber) {
+	syslog(LOG_WARNING, "appear to have received our own echo-reply!");
 	return;
+    }
 
     /* Reset the number of outstanding echo frames */
     lcp_echos_pending = 0;
@@ -1587,8 +1590,8 @@ LcpSendEchoRequest (f)
  * Detect the failure of the peer at this point.
  */
     if (lcp_echo_fails != 0) {
-        if (lcp_echos_pending++ >= lcp_echo_fails) {
-            LcpLinkFailure(f);
+        if ((int) lcp_echos_pending++ >= (int) lcp_echo_fails) {
+	    LcpLinkFailure (f);
 	    lcp_echos_pending = 0;
 	}
     }
