@@ -381,7 +381,7 @@ main(argc, argv)
 
 #ifdef TIOCSCTTY
     /* set device to be controlling tty */
-    if (!default_device && ioctl(fd, TIOCSCTTY) < 0) {
+    if (!default_device && ioctl(fd, TIOCSCTTY, 1) < 0) {
 	syslog(LOG_ERR, "ioctl(TIOCSCTTY): %m");
 	die(1);
     }
