@@ -5,7 +5,7 @@
   fcstab and some ideas nicked from if_ppp.c from cmu.
   See copyright notice in if_ppp.h and NOTES
 
-  $Id: ppp_async.c,v 1.8 1994/04/18 04:44:16 paulus Exp $
+  $Id: ppp_async.c,v 1.9 1994/05/27 00:40:02 paulus Exp $
 */
 
 #include <sys/types.h>
@@ -29,7 +29,6 @@
 #include <net/if.h>
 #include <netinet/in.h>
 
-#include <net/if_ppp.h>
 #include <net/ppp_str.h>
 
 #ifdef	DEBUGS
@@ -183,8 +182,7 @@ ppp_async_open(q, dev, flag, sflag)
 	    | PAI_FLAGS_RCV_COMPPROT;
 	p->pai_asyncmap[0] = 0xffffffff;	/* default async map */
 	p->pai_asyncmap[3] = 0x60000000;	/* escape 7d, 7e */
-	p->pai_buffsize = PPP_MTU + sizeof(struct ppp_header) + 
-	    sizeof(u_short);	/* how big a buffer to allocate */
+	p->pai_buffsize = PPP_MTU + PPP_HDRLEN + PPP_FCSLEN;
     }
     else {
 	p = (PAI *) q->q_ptr;
@@ -293,7 +291,7 @@ ppp_async_wput(q, mp)
 	    x = *(int *) mp->b_cont->b_rptr;
 	    if (x < PPP_MTU)
 		x = PPP_MTU;
-	    x += sizeof(struct ppp_header) + sizeof(u_short);
+	    x += PPP_HDRLEN + PPP_FCSLEN;
 	    if (x > 4096) {	/* couldn't allocb something this big */
 		i->ioc_error = EINVAL;
 		goto iocnak;
@@ -305,8 +303,7 @@ ppp_async_wput(q, mp)
 	case SIOCGIFMRU :
 	    if ((mp->b_cont = allocb(sizeof(int), BPRI_MED)) != NULL) {
 		*(int *) mp->b_cont->b_wptr = 
-		    p->pai_buffsize - (sizeof(struct ppp_header) + 
-				       sizeof(u_short));
+		    p->pai_buffsize - (PPP_HDRLEN + PPP_FCSLEN);
 		mp->b_cont->b_wptr += i->ioc_count  = sizeof(int);
 		goto iocack;
 	    }
@@ -663,12 +660,12 @@ ppp_async_rsrv(q)
 			continue;
 		    }
 
-		    if (p->pai_buffcount > sizeof(u_short)) { /* discard FCS */
-			adjmsg(p->pai_buffer, -sizeof(u_short));
-			p->pai_buffcount -= sizeof(u_short);
+		    if (p->pai_buffcount > PPP_FCSLEN) { /* discard FCS */
+			adjmsg(p->pai_buffer, -PPP_FCSLEN);
+			p->pai_buffcount -= PPP_FCSLEN;
 		    }
 
-		    if (p->pai_buffcount < sizeof(struct ppp_header)) {
+		    if (p->pai_buffcount < PPP_HDRLEN) {
 			if (p->pai_buffcount) {
 			    INPUT_ERROR(q);
 			    DLOG("ppp_async: short input packet (%d)\n",
@@ -704,14 +701,13 @@ ppp_async_rsrv(q)
 		/* here c != PPP_FLAG */
 		if (p->pai_flags & PAI_FLAGS_FLUSH)
 		    continue;			/* skipping chars */
-		if (c == PPP_ESCAPE) {
-		    p->pai_flags |= PAI_FLAGS_ESCAPED;
-		    continue;
-		}
 	
 		if (p->pai_flags & PAI_FLAGS_ESCAPED) {
 		    p->pai_flags &= ~PAI_FLAGS_ESCAPED; /* clear esc flag */
 		    c ^= PPP_TRANS;			/* xor's are cool */
+		} else if (c == PPP_ESCAPE) {
+		    p->pai_flags |= PAI_FLAGS_ESCAPED;
+		    continue;
 		}
 	
 		/* here we check to see if we have a buffer.
