@@ -6,7 +6,7 @@
   Copyright (C) 1990  Brad K. Clements, All Rights Reserved
   See copyright notice in NOTES
 
-  $Id: ppp_if.c,v 1.8 1994/05/27 00:40:23 paulus Exp $
+  $Id: ppp_if.c,v 1.9 1994/06/09 01:39:47 paulus Exp $
 */
 
 #define	VJC	1
@@ -499,9 +499,10 @@ ppp_if_rsrv(q)
     int dlen;
 #endif
     register PII *p;
-    struct mbuf	*mb1, *mb2,*mbtail;
+    struct mbuf	*mb1, *mb2, *mbtail;
     int	len, xlen, count, s;
     u_char *rptr;
+    int address, control;
 
     p = (PII *) q->q_ptr;
 
@@ -533,6 +534,8 @@ ppp_if_rsrv(q)
 		freemsg(mp);
 		continue;				
 	    }
+	    address = PPP_ADDRESS(mp->b_rptr);
+	    control = PPP_CONTROL(mp->b_rptr);
 
 	    xlen = MIN(len, MAX_VJHDR + PPP_HDRLEN);
 
@@ -569,6 +572,9 @@ ppp_if_rsrv(q)
 	     * Set up the protocol field, in case this message
 	     * gets put back on the queue.
 	     */
+	    mp->b_rptr[0] = address;
+	    mp->b_rptr[1] = control;
+	    mp->b_rptr[2] = 0;
 	    mp->b_rptr[3] = PPP_IP;
 	    DLOG("ppp_if%d: VJC decompressed\n", p - pii);
 	    break;
