@@ -371,7 +371,7 @@ pppread(tp, uio, flag)
 	return 0;
     s = splimp();
     while (sc->sc_inq.ifq_head == NULL && tp->t_line == PPPDISC) {
-	if (tp->t_state & TS_ASYNC) {
+	if (tp->t_state & (TS_ASYNC | TS_NBIO)) {
 	    splx(s);
 	    return (EWOULDBLOCK);
 	}
@@ -656,7 +656,7 @@ pppoutput(ifp, m0, dst)
 	 * put the packet on the fastq instead.
 	 */
 	if ((ip = mtod(m0, struct ip *))->ip_p == IPPROTO_TCP) {
-	    register int p = ((int *)ip)[ip->ip_hl];
+	    register int p = ntohl(((int *)ip)[ip->ip_hl]);
 	    if (INTERACTIVE(p & 0xffff) || INTERACTIVE(p >> 16))
 		ifq = &sc->sc_fastq;
 	}
@@ -1348,16 +1348,15 @@ pppinput(c, tp)
 	return;
     }
 
-    if (c == PPP_ESCAPE) {
-	sc->sc_flags |= SC_ESCAPED;
-	return;
-    }
     if (c < 0x20 && (sc->sc_rasyncmap & (1 << c)))
 	return;
 
     if (sc->sc_flags & SC_ESCAPED) {
 	sc->sc_flags &= ~SC_ESCAPED;
 	c ^= PPP_TRANS;
+    } else if (c == PPP_ESCAPE) {
+	sc->sc_flags |= SC_ESCAPED;
+	return;
     }
 
     /*
